from typing import NamedTuple, Optional, Callable, List
from datetime import datetime


class Summary(NamedTuple):
    wall_time: float
    step: int
    value: float
    tag: str


def wall_time_in_range(
    *, min: Optional[datetime], max: Optional[datetime]
) -> Callable[[Summary], bool]:
    def predicate(x: Summary) -> bool:
        if min is not None and min.timestamp() > x.wall_time:
            return False

        if max is not None and max.timestamp() < x.wall_time:
            return False

        return True

    return predicate


def tag(tag: str) -> Callable[[Summary], bool]:
    return lambda x: x.tag == tag


def tags(tags: List[str]) -> Callable[[Summary], bool]:
    return lambda x: x.tag in tags



from typing import Iterable, Dict, List
import os
from glob import glob
from collections import defaultdict

import tensorflow as tf  # type: ignore
from tensorflow.core.util import event_pb2  # type: ignore

from tensorboard_reporter.summary import Summary


def load_summaries(run_dir: str) -> Iterable[Summary]:
    event_files = glob(os.path.join(run_dir, "*.tfevents.*"))

    it = filter(
        lambda x: x.WhichOneof("what") == "summary",
        map(
            lambda x: event_pb2.Event.FromString(x.numpy()),  # type: ignore
            tf.data.TFRecordDataset(event_files),
        ),
    )

    for elem in it:
        for value in elem.summary.value:
            if value.WhichOneof("value") == "simple_value":
                yield Summary(
                    wall_time=elem.wall_time,
                    step=elem.step,
                    tag=value.tag,
                    value=value.simple_value,
                )
            elif value.WhichOneof("value") == "tensor":
                try:
                    yield Summary(
                        wall_time=elem.wall_time,
                        step=elem.step,
                        tag=value.tag,
                        value=float(tf.make_ndarray(value.tensor)),
                    )
                except ValueError:
                    continue


def group_by_tag(summaries: Iterable[Summary]) -> Dict[str, List[Summary]]:
    tags: Dict[str, List[Summary]] = defaultdict(list)

    for summary in summaries:
        tags[summary.tag].append(summary)

    return tags



from typing import List, Optional

import numpy as np  # type: ignore

from tensorboard_reporter.summary import Summary


def current_mean(summaries: List[Summary], fraction: float = 0.1) -> Optional[float]:
    if len(summaries) == 0:
        return None

    last_part = max(1, int(len(summaries) * fraction))

    return float(np.mean(list(map(lambda x: x.value, summaries[-last_part:]))))
